@@ -1,4 +1,4 @@
-# $Id: Mpp.pm,v 1.25 2013/08/19 06:59:10 pfeiffer Exp $
+# $Id: Mpp.pm,v 1.28 2016/09/28 20:36:49 pfeiffer Exp $
 
 =head1 NAME
 
@@ -137,7 +137,7 @@ sub log($@);
 
 my $invocation = join_with_protection($0, @ARGV);
 
-for( qw(/usr/xpg4/bin/sh /sbin/xpg4/sh /bin/sh) ) {
+for( </{usr/xpg4/bin,sbin/xpg4,bin,usr/bin}/sh> ) {
   if( -x ) {
     $ENV{SHELL} = $_;		# Always use a hopefully Posix shell.
     last;
@@ -146,25 +146,32 @@ for( qw(/usr/xpg4/bin/sh /sbin/xpg4/sh /bin/sh) ) {
 delete $ENV{PWD};		# This is dangerous.
 
 our $indent_level = 0;		# Indentation level in the log output.
-our $keep_going = 0;		# -k specified.
 my $logfile;			# Other than default log file.
 our $parallel_make = 0;		# True if we're in parallel make mode.
 our $profile = 0;		# Log messages about execution times.
 our $verbose;
 our $quiet_flag;		# Default to printing informational messages.
 
+# Arguments to print may be split and get concatenated.  Trailing \n optional.
+# 3 cases:
+# "xxx: ..." probably file:lineno:
+# ": ..." same but file:lineno was not available at call time, add $progname on STDERR
+# any other, add $progname: on STDERR
 $SIG{__WARN__} = sub {
-  my $level = ($_[0] =~ /^(?:error()|info): /s) ? '' : 'warning: ';
-  my $error = defined $1;
+  my( $str, $pos ) = ($_[0], '');
+  $str =~ s/^: //g or		# leading ': '
+    $str =~ /^(?!error:|warning:|info:)\S*: /gc; # GNU style?
+  $pos = pos $str;
+  $str =~ s/\G(?!error: |warning: |info: )/warning: /; # add warning
+  $str =~ s/\A(\S+?:\d+)\((\S+?\))(: (?:error|warning|info): )/$1$3(dir $2 /;
   if( $log_level == 2 ) {
     &Mpp::log() unless defined $logfh; # Only open the file.
-    print $logfh "*** $level$_[0]";
+    print $logfh "*** $str";
   }
-  print STDERR "$progname: $level$_[0]" if $error or $warn_level;
+  print STDERR $pos ? '' : "$progname: ", $str if $warn_level or $str =~ /\berror: /;
 };
 
 
-
 =head2 Mpp::log
 
   Mpp::log KEY => {object | array of objects | string} ...
@@ -325,36 +332,6 @@ sub print_profile_end {
   print "$progname: End \@" . &$hires_time() . " $_[0]\n";
 }
 
-=head2 print_error
-
-  print_error "message", ...;
-
-Prints an error message, with the program name prefixed.  For any arg which is
-a reference, $arg->name is printed.
-
-If any filename is printed, it should have be quoted as in `filename' or
-`filename:lineno:' at BOL, so that IDEs can parse it.
-
-=cut
-
-sub print_error {
-  my( $log, $stderr ) = $log_level && '*** ';
-  my $str = '';
-  if( $_[0] =~ /^error()/ || $_[0] !~ /^\S+?:/ ) { # No name?
-    $stderr = "$progname: ";
-    $str = 'error: ' unless defined $1;
-  }
-  $str .= ref() ? $_->name : $_
-    for @_;
-  $str .= "\n" if $str !~ /\n\z/;
-  print STDERR $stderr ? $stderr . $str : $str;
-  if( $log_level == 2 ) {
-    &Mpp::log() unless defined $logfh; # Only open the file.
-    print $logfh $log . $str;
-  }
-  &flush_log;
-}
-
 
 sub perform(@) {
   #my @targets = @_;		# Arguments passed to build and then wait_for.
@@ -431,7 +408,11 @@ signal propagation.$orig Stopped}
   }
 
   print "$progname: Ending \@" . &$hires_time() . "\n" if $profile;
-  print_error $error_message if $error_message;
+  if( $error_message ) {
+    $error_message =~ s/^(\S+: )?(?!error: )/($1 ? $1 : '') . 'error: '/e;
+    $error_message =~ s/\s*\Z/\n/;
+    warn $error_message;
+  }
 
   if( $Mpp::loop ) {
     &Mpp::File::update_build_infos;
@@ -446,7 +427,7 @@ signal propagation.$orig Stopped}
       $failed_count = 0;
     $Mpp::stop = 1;
     my @dirs = $Mpp::File::root;
-    Mpp::File::touched_filesystem;
+    &touched_filesystem;
     while( @dirs ) {
       my $dinfo = pop @dirs;
       next if exists $dinfo->{xREPOSITORY};
@@ -456,7 +437,7 @@ signal propagation.$orig Stopped}
                                 # Don't traverse it if it's a symbolic link,
                                 # because then we'll find the same files twice.
 	} else {
-	  Mpp::File::may_have_changed( $_ );
+	  may_have_changed $_;
 	  delete $_->{BUILD_HANDLE}; # Also forget we may have already built it.
 	}
       }
@@ -472,11 +453,11 @@ signal propagation.$orig Stopped}
 
 our @common_opts =
   (
-    ['k', qr/keep[-_]?going/, \$keep_going],
+    ['k', qr/keep[-_]?going/, \our $keep_going],
 
     [undef, qr/log(?:[-_]?file)?/, \$logfile, 1],
 
-    ['n', qr/(?:just[-_]?print|dry[-_]?run|recon)/, \$Mpp::dry_run],
+    ['n', qr/(?:just[-_]?print|dry[-_]?run|recon)/, \our $dry_run],
     [undef, qr/no[-_]?log/, \$log_level, undef, 0], # Turn off logging.
     [undef, qr/no[-_]?print[-_]?directory/, \$print_directory, 0, undef],
     [undef, qr/no[-_]?warn/, \$warn_level, undef, 0],
