@@ -1,4 +1,4 @@
-# $Id: Esql.pm,v 1.23 2014/01/15 21:32:38 pfeiffer Exp $
+# $Id: Esql.pm,v 1.24 2014/07/03 21:13:10 pfeiffer Exp $
 
 =head1 NAME
 
@@ -119,7 +119,7 @@ sub parse_arg {
       }
     } elsif( $_[0]{SUFFIXES} && Mpp::is_windows ? /(?:^|[\\\/])[^.]+$/ : /(?:^|\/)[^.]+$/ ) {
       for my $suffix ( @{$_[0]{SUFFIXES}} ) {
-	if( Mpp::File::exists_or_can_be_built file_info "$_$suffix", $_[0]{RULE}{MAKEFILE}{CWD} ) {
+	if( Mpp::File::exists_or_can_be_built file_info( "$_$suffix", $_[0]{RULE}{MAKEFILE}{CWD} ), 0 ) {
 	  push @$files, "$_$suffix";
 	  last;
 	}
