@@ -1,4 +1,4 @@
-# $Id: Glob.pm,v 1.45 2013/05/09 08:41:00 pfeiffer Exp $
+# $Id: Glob.pm,v 1.49 2016/09/28 20:36:49 pfeiffer Exp $
 
 package Mpp::Glob;
 
@@ -10,7 +10,7 @@ require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(chdir);	# Force our caller to use our chdir sub that
 				# we inherit from Mpp::File.
-our @EXPORT_OK = qw(zglob zglob_fileinfo $allow_dot_files wildcard_do);
+our @EXPORT_OK = qw(zglob zglob_fileinfo wildcard_do);
 
 use strict;
 
@@ -97,11 +97,7 @@ sub zglob_fileinfo_atleastone {
 sub zglob_fileinfo {
   local $_ = $_[0];		# Access the filename or wildcard.
   m@^/\*\*@ and die "Refusing to expand /** as wildcard--you don't want to search every directory in the file system\n";
-  my $startdir = $_[1] || $CWD_INFO;
-				# Get the current directory.
-  my $dont_follow_soft = $_[2];
-  my $phony = $_[3] ? 1 : 0;
-  my $stale = $_[4] ? 1 : 0;
+  my( undef, $startdir, $dont_follow_soft, $phony, $stale ) = @_;
 
   my $is_wildcard = 0;		# We haven't seen a wildcard yet.
 
@@ -117,25 +113,25 @@ sub zglob_fileinfo {
 				# strip out the leading /.
   my @pieces = split /\/+/;	# Get the pieces of the filename, and
 				# examine each one of them separately.
-  my @new_candidates = ($startdir); # Directories that are possible.  At first,
+  my @new_candidates = ($startdir || $CWD_INFO); # Directories that are possible.  At first,
 				# there is only the starting directory.
 
-  while ($_ = shift @pieces) {
+  while( $_ = shift @pieces ) {
     my @candidate_dirs = $dont_follow_soft ?
-      grep($_->{DIRCONTENTS} && !Mpp::File::is_symbolic_link( $_ ) ||
-	   is_or_will_be_dir( $_ ),
-	   @new_candidates) :
-      grep($_->{DIRCONTENTS} || is_or_will_be_dir( $_ ), @new_candidates);
+      grep( $_->{DIRCONTENTS} && !Mpp::File::is_symbolic_link( $_ ) ||
+	is_or_will_be_dir( $_ ),
+	@new_candidates ) :
+      grep( $_->{DIRCONTENTS} || is_or_will_be_dir( $_ ), @new_candidates );
 				# Discard everything that isn't a directory,
 				# since we have to look for files in it.
 				# (Note that we will return files that are
 				# in directories that don't exist yet.)
 
-    if ($Mpp::implicitly_load_makefiles) { # Should wildcards trigger loading?
+    if( $Mpp::implicitly_load_makefiles ) { # Should wildcards trigger loading?
 				# We have to do this before scanning the
 				# directory, since loading the makefile
 				# may make extra files appear.
-      Mpp::Makefile::implicitly_load($_) for @candidate_dirs;
+      exists $_->{MAKEINFO} or Mpp::Makefile::implicitly_load( $_ ) for @candidate_dirs;
     }
 
 
@@ -154,7 +150,7 @@ sub zglob_fileinfo {
 #
 # The remaining wildcards match only files within these directories.
 #
-    my @phony_expr = (@pieces ? () : ($phony, $stale, 1)); # Set $no_last_chance
+    my @phony_expr = @pieces ? (0) : ($phony, $stale, 1); # Set $no_last_chance
     if( /[[?*]/ ) { # Was there actually a wildcard?
       my $re = wild_to_regex( $_, 3 ); # Convert to a regular expression.
       my $allow_dotfiles = $allow_dot_files || ord( '.' ) == ord;
@@ -169,7 +165,7 @@ sub zglob_fileinfo {
 	while( my( $fname, $finfo ) = each %dircontents ) {
 	  next unless $allow_dotfiles || ord( $fname ) != ord '.'
 	    and $fname =~ $re;
-	  Mpp::File::exists_or_can_be_built $finfo, @phony_expr and # File must exist, or
+	  Mpp::File::exists_or_can_be_built $finfo, @phony_expr and # File "exist"s, as per phony, etc.
 	    push @new_candidates, $finfo;
 	}
       }
@@ -476,7 +472,7 @@ dependencies.  Usage:
   wildcard_do {
     my( $finfo, $plain ) = @_;
     ...
-  } [\1,] @wildcards;
+  } [\1|\2,] @wildcards;
 
 The block is called once for each file that matches the wildcards.  If at some
 later time, files which match the wildcard are created (or we find rules to
@@ -485,7 +481,8 @@ Mpp::File::publish, which is called automatically whenever a file which didn't
 use to exist now exists, or whenever a build rule is specified for a file
 which does not currently exist.)
 
-An optional reference as 2nd parameter means this is from a last chance rule.
+An optional reference as 2nd parameter, if it is \1 means this is from a last
+chance rule.  If it is \2 do not return phonies.
 
 You can specify non-wildcards as arguments to wildcard_do.  In this case, the
 block is called once for each of the files explicitly listed, even if they
@@ -530,7 +527,9 @@ this case.
 =cut
 
 sub wildcard_do(&@) {
-  my( $subr, $last_chance ) = splice @_, 0, ref( $_[1] ) ? 2 : 1;
+  my( $subr, $flags ) = splice @_, 0, ref( $_[1] ) ? 2 : 1;
+  my ( $last_chance, $no_phony ) = ($$flags & 1, $$flags & 2 ? 0 : undef)
+    if $flags;
   my $member = $last_chance ? 'LAST_CHANCE' : 'WILDCARD_DO';
 
 #
@@ -577,14 +576,14 @@ sub wildcard_do(&@) {
 # a regular expression that matches:
 #
     my $idx = 0;
-    while ($idx < @file_pieces) {
-      if ($file_pieces[$idx] eq '.') { # Remove useless './' components
-	splice(@file_pieces, $idx, 1); # (since they will mess up the regex).
+    while( $idx < @file_pieces ) {
+      if( $file_pieces[$idx] eq '.' ) { # Remove useless './' components
+	splice @file_pieces, $idx, 1; # (since they will mess up the regex).
 	next;			# Go back to top without incrementing idx.
       }
 
-      if ($file_pieces[$idx] eq '..') { # At least give a warning message
-	warn "$0: .. is not supported after a wildcard
+      if( $file_pieces[$idx] eq '..' ) { # At least give a warning message
+	warn ".. is not supported after a wildcard
   in the wildcard expression \"$filename\".
   This will only match existing files.\n";
 				# Let user know this will not do what he thinks.
@@ -596,7 +595,7 @@ sub wildcard_do(&@) {
     local $_ = join '/', @file_pieces;
     $need_dir ||= 1 if $file_pieces[-1] =~ /\*\*/;
     unless( $last_chance ) {
-      for my $finfo ( zglob_fileinfo $_, $dirinfo, 1 ) {
+      for my $finfo ( zglob_fileinfo $_, $dirinfo, 1, $no_phony ) {
 	next if $need_dir && !Mpp::File::is_or_will_be_dir $finfo;
 	$finfo->{PUBLISHED}=2 if $Mpp::rm_stale_files;
 				# Don't also call $subr later if it looks like
