@@ -1,10 +1,10 @@
-# $Id: Makefile.pm,v 1.175 2013/12/01 19:24:04 pfeiffer Exp $
+# $Id: Makefile.pm,v 1.184 2016/09/28 20:36:49 pfeiffer Exp $
 package Mpp::Makefile;
 
 use Mpp::Glob qw(wildcard_do);
 use Mpp::Event qw(wait_for);
 use Mpp::Text qw(rfind_unquoted find_unquoted split_on_whitespace split_on_colon
-		 unquote unquote_split_on_whitespace requote hash_neq);
+		 unquote unquote_split_on_whitespace hash_neq);
 use Mpp::Subs;
 use Mpp::Cmds ();
 use Mpp::File;
@@ -270,7 +270,7 @@ sub expand_expression {
   $expr = expand_text $self, $expr, $makefile_line unless $expanded;
   if( $expr =~ s/^&(?=.)// ) { # & alone is a silly variable
     my( $cmd, @args ) = unquote_split_on_whitespace $expr;
-    local $Mpp::Subs::rule = { MAKEFILE => $self, RULE_SOURCE => $makefile_line };
+    local $Mpp::Subs::rule = bless { MAKEFILE => $self, RULE_SOURCE => $makefile_line }, 'Mpp::Rule';
     open my $ofh, '>&STDOUT' or die;
     close STDOUT;
     open STDOUT, '>:crlf', \($result = '') or die $!; # \$result gives undef warning in 5.8.1 - 5.10.0
@@ -296,7 +296,7 @@ sub expand_expression {
 				# Substitution reference ('x:%.o=%.c' or 'x:o=c')?
     my $pct = 0 <= index $2, '%'; # Is it the 1st form?
     $result = join ' ',
-      Mpp::Text::pattern_substitution $pct ? ($2, $3) : ("%$2", "%$3"),
+      Mpp::Text::pattern_substitution $pct ? my @copy = ($2, $3) : ("%$2", "%$3"), # copy in case next line recurses here, inner sticks after that
 	split_on_whitespace expand_variable( $self, $1, $makefile_line );
   } else {			# Must be a vanilla variable to expand.
     return expand_variable( $self, $expr, $makefile_line );
@@ -308,7 +308,7 @@ sub expand_expression {
       if Mpp::DEBUG;
     $result;
   } else {
-    warn "expression `$expr' expanded to an undefined value at `$makefile_line'.\n";
+    warn "$makefile_line: expression `$expr' expanded to an undefined value.\n";
     '';
   }
 }
@@ -340,7 +340,9 @@ our $rule_include; # undef unless reading an :include .d file. 1 before rule exe
 # d) The mode is one of
 #	false	simply expand the variable
 #	1	return (expand_flag, unexpanded_value)
-#	other	return whether the variable is defined
+#	2	return whether the variable is defined
+#	3	skip private vars
+#	4	start private and package vars
 #
 sub expand_variable {
   my( $self, $var, $makefile_line, $mode ) = @_; # Name the arguments.
@@ -358,8 +360,13 @@ sub expand_variable {
 				# mechanism, so we can't just loop through
 				# these:
 
+    if( defined $mode && $mode > 2 ) {
+      undef $mode;		# Do normal var expansion
+      goto "attempt$_[3]";	# $mode we just nuked.
+    }
+
 # 1st attempt:
-    if( $var =~ /^\d+$/ || exists $Mpp::Subs::perl_unfriendly_symbols{$var} ) { # Is it one of the call
+    if( length( $var ) < 3 and $var =~ /^\d+$/ || exists $Mpp::Subs::perl_unfriendly_symbols{$var} ) { # Is it one of the call
 				# or 1-char symbols like '$(11)' or '$@' that conflict with Perl variables?
 				# The call fn may eval more than got passed.  These can't be per target or global.
       if( ref $Mpp::Subs::perl_unfriendly_symbols{$var} ) {
@@ -391,7 +398,7 @@ sub expand_variable {
 	last;
     }
 
-# 3rd attempt:
+  attempt3:
     defined( $result = ${"$self->{PACKAGE}::$var"} ) and
 				# Get it from the makefile.
       $varref = \${"$self->{PACKAGE}::$var"},
@@ -444,7 +451,10 @@ sub expand_variable {
     $reexpand = $reexpand->{VAR_REEXPAND}{$var};
 
   if( !$mode ) {
-    defined $result or $result = ''; # Variable not found--make it defined.
+    unless( defined $result ) {	# Variable not found--make it defined.
+      $result = '';
+      warn $makefile_line || '', ": undefined variable '$var'\n" if $Mpp::warn_undef_var and $var !~ /^makepp_/;
+    }
     if( $reexpand ) {
       Mpp::log VAR_EXPAND => $var, $result, $makefile_line || ''
 	if Mpp::DEBUG;
@@ -483,7 +493,7 @@ sub find_makefile_in {
   for( @root_makefiles, qw(Makeppfile Makeppfile.mk), $_[1] ? () : qw(makefile Makefile) ) {
     my $trial_makefileinfo = file_info $_, $dir;
     return $trial_makefileinfo
-      if Mpp::File::exists_or_can_be_built $trial_makefileinfo;
+      if Mpp::File::exists_or_can_be_built $trial_makefileinfo, 0;
   }
 }
 
@@ -491,17 +501,13 @@ sub find_makefile_in {
 # This subroutine is called when we have discovered a new directory and we
 # want to load a makefile implicitly from it, if we haven't already.
 #
-# Argument: the Mpp::File structure for the relevant directory.
+# Argument: the Mpp::File structure for the relevant directory.  This
+# directory must not already have a {MAKEINFO} member!
 #
 sub implicitly_load {
-  $Mpp::implicitly_load_makefiles or return;
-				# Don't do anything if we don't implicitly
-				# load makefiles from directories.
-
   my $dirinfo = $_[0];
 
-  exists $dirinfo->{MAKEINFO} and return;
-				# Already tried to load something.
+  $dirinfo->{MAKEINFO} = undef;	# Remember that we tried to load something.
   Mpp::File::is_writable( $dirinfo ) ||	# Directory already exists?
     !exists $dirinfo->{xEXISTS} && is_or_will_be_dir( $dirinfo ) &&
     exists $dirinfo->{ALTERNATE_VERSIONS}
@@ -514,7 +520,7 @@ sub implicitly_load {
 # See if this directory or any of its parents is marked for no implicit
 # loading.
 #
-  for (my $pdirinfo = $dirinfo; $pdirinfo; $pdirinfo = $pdirinfo->{'..'} || '') {
+  for( my $pdirinfo = $dirinfo; $pdirinfo; $pdirinfo = $pdirinfo->{'..'} || '' ) {
     exists $pdirinfo->{xNO_IMPLICIT_LOAD} and return;
   }
 
@@ -525,13 +531,8 @@ sub implicitly_load {
 	      \%Mpp::global_ENV,
 	      $Mpp::implicit_load_makeppfile_only) };
 				# Try to load the makefile.
-  $dirinfo->{MAKEINFO} ||= undef;
-				# Remember that we tried to load something,
-				# even if we failed.
-  if ($@ &&			# Some error?
-      $@ !~ /can\'t find a makefile in directory/) { # Unrecognized error?
-    die $@;			# Resignal the error.
-  }
+  die $@ if			# Resignal unrecognized error.
+    $@ && $@ !~ /can\'t find a makefile in directory/;
 }
 
 =head2 cleanup_vars
@@ -1016,7 +1017,7 @@ sub assign {
                                 # This allows a user to override buggy
                                 # read-only makefiles.
 
-  warn "MAKE redefined at `$makefile_line', recursive make won't work as expected\n"
+  warn "$makefile_line: MAKE redefined, recursive make won't work as expected\n"
     if $name eq 'MAKE';
 
   if( $type == ord '?' ) {
@@ -1048,7 +1049,7 @@ sub assign {
     }
   }
 
-  if( !$type ) {	# Plain assignment?
+  if( !$type ) {		# Plain assignment?
 
     $$varref = $value;
     $reexpand = 1;		# Remember to expand this variable's contents
@@ -1058,6 +1059,12 @@ sub assign {
 
     $$varref = expand_text $self, $value, $makefile_line;
 
+  } elsif( $type == ord ';' ) { # Postponed evaluation once?
+
+    $$varref = $value;
+    $reexpand = 2;		# Remember to expand this variable's contents
+				# when it's invoked, and then change to :=.
+
   } elsif( $type == ord '+' ) { # Append?
 
     ($reexpand, $$varref) = expand_variable $self, $name, $makefile_line, 1;
@@ -1073,12 +1080,6 @@ sub assign {
 				# Expand the RHS if it was set with :=
 				# previously.
 
-  } elsif( $type == ord ';' ) { # Postponed evaluation once?
-
-    $$varref = $value;
-    $reexpand = 2;		# Remember to expand this variable's contents
-				# when it's invoked, and then change to :=.
-
   } elsif( $type == ord '&' ) { # Prepend?
 
     ($reexpand, $$varref) = expand_variable $self, $name, $makefile_line, 1;
@@ -1122,6 +1123,7 @@ sub assign {
 
 #
 # Grok a potential assignment statement.  Arguments:
+# $_ the assignment, which may be modified.
 # a) The makefile.
 # b) The makefile line number (for error messages).
 #  ) $_ is the implicit argument containing the assignment.
@@ -1165,9 +1167,9 @@ sub grok_assignment {
     return unless defined $_[2];
   }
 
+  $keyword->{define} ? s/[ \t]*\n\Z// : s/\s+$//;
   my $var_value = substr $_, $_[2] + 1;
   $var_value =~ s/^\s+//;
-  $var_value =~ s/\s+$//;
 
   my @list;
   if( $var_name =~ /:/ and (@list = split_on_colon $var_name) > 1 ) {
@@ -1312,12 +1314,11 @@ sub grok_rule {
 	 $last_line_was_blank) ||
 	($whitespace_len >= 8 &&
 	 $whitespace_len <= $target_whitespace_len)) {
-      substr $_, 0, 0, ' ' x $target_whitespace_len;
-				# Put the whitespace back (in case it's the
-				# next target).
+      substr $_, 0, 0, ' ' x $whitespace_len;
+				# Put the whitespace back (in case it's the next target).
       last;			# We've found the end of this rule.
     }
-    if (/^:\s*((?:build[-_]?c(?:ache|heck)|dispatch|env(?:ironment)?|foreach|multiple[-_]?rules[-_]?ok|parser|s(?:ignature|canner|martscan)|quickscan|last[-_]?chance)\b.*)/ ) {
+    if( /^:\s*((?:build[-_]?c(?:ache|heck)|dispatch|env(?:ironment)?|foreach|multiple[-_]?rules[-_]?ok|parser|s(?:ignature|canner|martscan)|quickscan|last[-_]?chance|no[-_]?phony)\b.*)/ ) {
 				# A colon modifier?
       push @after_colon, $1;
       if( (my $i = find_unquoted $after_colon[-1], '#') > 0 ) {
@@ -1327,30 +1328,26 @@ sub grok_rule {
       $action .= $_;		# Must be an action for the rule.
     }
 
-    $first_action_indent ||= $whitespace_len; # If this was the first line,
-				# remember its indentation.
+    $first_action_indent ||= $whitespace_len; # Remember first line's indentation.
     $last_line_was_blank = 0;	# This line was not blank.
   }
 
   unshift @hold_lines, $_ if $_; # We read too far, so put this line back.
+  Mpp::log RULE_GROK => $skipping ? 'skip' : 'grok', $makefile_line, $target_string, join( ' :', '', @after_colon ), $action
+    if Mpp::DEBUG;
   return if $skipping;
 
 #
 # Pull off the : modifiers.
 #
-  my( $foreach, $signature, $signature_name, $signature_override, $build_check, $build_cache );
-  my( $lexer, $parser );
-  my $conditional_scanning;
-  my $multiple_rules_ok;
-  my $last_chance_rule;
-  my $dispatch;
-  my $env_dep_str;
-  my $include;
+  my( $foreach, $signature, $signature_name, $signature_override, $build_check, $build_cache, $lexer, $parser,
+      $conditional_scanning, $multiple_rules_ok, $last_chance_rule, $dispatch, $env_dep_str, $include, @no_phony );
 
   while( @after_colon > 1 ) {	# Anything left?
     if( $after_colon[-1] =~ /^\s*foreach\s+(.*?)\s*$/ ) {
       $foreach and die "$makefile_line: multiple :foreach clauses\n";
       $foreach = expand_text $self, $1, $makefile_line;
+      @no_phony = \2;
     } elsif( $after_colon[-1] =~ /^\s*build[-_]?c(?:ache|heck())\s+(.*?)\s*$/ ) { # Build cache or check?
       die "$makefile_line: multiple :build_" . (defined $1 ? 'check' : 'cache') . " clauses\n"
 	if defined $1 ? $build_check : $build_cache;
@@ -1412,6 +1409,8 @@ sub grok_rule {
       $dispatch = $1;
     } elsif( $after_colon[-1] =~ /^\s*last[-_]?chance/ ) {
       $last_chance_rule = 1;
+    } elsif( $after_colon[-1] =~ /^\s*no[-_]?phony/ ) {
+      @no_phony = \2;
     } elsif( $after_colon[-1] =~ /^\s*include\s+(.*?)\s*$/ ) {
       $include = $1;
     } else {			# Something we don't recognize?
@@ -1525,7 +1524,7 @@ sub grok_rule {
     }
   };
 
-  if ($foreach) {		# Is there a foreach clause?
+  if( $foreach ) {		# Is there a foreach clause?
     die "$makefile_line: Combining \":foreach\" and \":last_chance\" is not supported.\n" if $last_chance_rule;
 ###### TODO: This needs to handle rules with no actions here, as well as
 ###### below where there's no :foreach clause.
@@ -1593,12 +1592,11 @@ sub grok_rule {
 				# a candidate for the first target in the file.
 	  $self->{FIRST_TARGET} ||= $tinfo;
 				# Remember what the first target is, in case
-				# no target was specified on the command
-				# line.
+				# no target was specified on the command line.
       }
       &$handle_include( $rule, $include ) if $include;
-    } unquote_split_on_whitespace expand_text $self, $foreach, $makefile_line; # End block called on every file that
-				# matches the wildcard.
+    } @no_phony,		# End block called on every file that matches the wildcard.
+      unquote_split_on_whitespace expand_text $self, $foreach, $makefile_line;
   } elsif(!defined $foreach) {	# it's not a foreach rule
 #
 # This rule is not a pattern rule.  If there is an action, then it's
@@ -1803,7 +1801,7 @@ sub read_block {
       my $end = $_[2] ? $line =~ s/^$re\s*(.*)//s : $re ? $line =~ /^$re\s*(.*)/s : undef;
       $code .= $line if defined wantarray;
       if( $end ) {		# Stop at a brace at the left margin or re.
-	warn "trailing cruft `" . substr( $1, 0, -1 ) . "' at `$makefile_name:$makefile_lineno'\n"
+	warn "$makefile_name:$makefile_lineno: trailing cruft `" . substr( $1, 0, -1 ) . "'\n"
 	  if $1 !~ /^$|^#/;
 	last;
       }
@@ -1892,7 +1890,7 @@ sub read_makefile {
     if( /^\s*(-?)\s*&(\w+)\s*(.*)/ && !$c_preprocess ) { # &Command at beginning of line?
       my( $ignore_error, $cmd ) = ($1, $2);
       my @args = unquote_split_on_whitespace expand_text( $self, $3, $makefile_line ) if length $3;
-      local $Mpp::Subs::rule = { MAKEFILE => $self, RULE_SOURCE => $makefile_line };
+      local $Mpp::Subs::rule = bless { MAKEFILE => $self, RULE_SOURCE => $makefile_line }, 'Mpp::Rule';
       eval {
 	chdir $self->{CWD};	# Make sure we're in the correct directory
 				# because some commands will expect this.
@@ -2236,7 +2234,7 @@ sub _read_makefile_line_stripped_1 {
 	undef $line;		# line has already been processed
 	last;
       } elsif( $line =~ /^\s*endif\s*(?:#|$)/ ) { # What the heck?
-	warn "empty conditional at `$makefile_name:$last_conditional_start'\n";
+	warn "$makefile_name:$last_conditional_start: empty conditional\n";
 	goto &_read_makefile_line_stripped_1; # Return a new line.
       } else {			# Reached then branch.
 	$totaltruthval ||= $truthval;
