@@ -1,4 +1,4 @@
-# $Id: Repository.pm,v 1.16 2013/08/19 06:38:27 pfeiffer Exp $
+# $Id: Repository.pm,v 1.18 2014/08/03 21:10:36 pfeiffer Exp $
 
 =head1 NAME
 
@@ -187,13 +187,13 @@ sub Mpp::Repository::get {
 
   if( $dest_finfo->{DIRCONTENTS} ) { # Is this a directory?
     if( !$src_finfo->{DIRCONTENTS} ) {
-      Mpp::print_error 'Directory `' . &absolute_filename . "' is in the way of a repository import.";
+      warn 'Directory `' . &absolute_filename . "' is in the way of a repository import.";
       ++$Mpp::failed_count;
       return 1;
     } elsif( &mkdir ) {	# Just make it if inexistent, don't soft link to it.
       return 0;
     } else {
-      Mpp::print_error 'Failed to make directory `' . &absolute_filename . "'--$!.";
+      warn 'Failed to make directory `' . &absolute_filename . "'--$!.";
       ++$Mpp::failed_count;
       return 1;
     }
@@ -201,10 +201,10 @@ sub Mpp::Repository::get {
 
   # Don't link to the repository if the source doesn't exist (even if it
   # can be built, because we'll build it locally instead).
-  return 0 unless exists_or_can_be_built_norecurse $src_finfo, undef, 1;
+  return 0 unless exists_or_can_be_built_norecurse $src_finfo, 0, 1;
 
   if( &dont_read or my $src = dont_read $src_finfo ) {
-    Mpp::print_error 'Cannot link ', $src_finfo, ' to ', $dest_finfo,
+    warn 'Cannot link ', $src_finfo, ' to ', $dest_finfo,
       ' because the ', $src ? 'latter' : 'former', ' is marked for dont-read';
     ++$Mpp::failed_count;
     return 1;			# Indicate failure.
@@ -269,7 +269,7 @@ sub Mpp::Repository::get {
       eval { &symlink };	# Make the link.
     }
     if ($@) {			# Did something go wrong?
-      Mpp::print_error 'Cannot link ', $src_finfo, ' to ', $dest_finfo, ":\n$@";
+      warn 'Cannot link ', $src_finfo, ' to ', $dest_finfo, ":\n$@";
       ++$Mpp::failed_count;
       return 1;			# Indicate failure.
     }
