@@ -1,4 +1,4 @@
-# $Id: Subs.pm,v 1.208 2013/12/01 19:25:41 pfeiffer Exp $
+# $Id: Subs.pm,v 1.213 2016/09/06 19:59:07 pfeiffer Exp $
 
 =head1 NAME
 
@@ -36,12 +36,11 @@ package Mpp::Subs;
 
 use strict qw(vars subs);
 
-use Mpp::Text qw(find_unquoted split_on_whitespace requote
-		unquote unquote_split_on_whitespace format_exec_args);
+use Mpp::Text qw(find_unquoted split_on_whitespace unquote unquote_split_on_whitespace format_exec_args);
 use Mpp::File;
 use Mpp::FileOpt;
-use Mpp::Event qw(wait_for when_done read_wait);
-use Mpp::Glob qw(zglob zglob_fileinfo);
+use Mpp::Event qw(wait_for when_done);
+use Mpp::Glob qw(zglob zglob_fileinfo wildcard_do);
 use Mpp::CommandParser;
 use Mpp::CommandParser::Gcc;
 
@@ -87,8 +86,8 @@ our $rule;
 #
 # TODO: is $ENV{INCLUDE} a reliable alternative on native Windows?  And if
 # ActiveState is to call MinGW gcc, must makepp translate directory names?
-our @system_include_dirs = grep -d, qw(/usr/local/include /usr/include);
-our @system_lib_dirs = grep -d, qw(/usr/local/lib /usr/lib /lib);
+our @system_include_dirs = grep -d && <$_/*>, qw(/usr/local/include /usr/include);
+our @system_lib_dirs = grep -d && <$_/*>, qw(/usr/local/lib /usr/lib /lib);
 
 sub p_gcc_compilation {
   shift;
@@ -485,10 +484,10 @@ sub f_find_program {
   foreach my $name ( split ' ', &arg) {
     if( $name =~ /\// || Mpp::is_windows > 1 && $name =~ /\\/ ) { # Either relative or absolute?
       my $finfo = path_file_info $name, $mkfile->{CWD};
-      my $exists = Mpp::File::exists_or_can_be_built $finfo;
+      my $exists = Mpp::File::exists_or_can_be_built $finfo, 0;
       if( Mpp::is_windows && $name !~ /\.exe$/ ) {
 	my( $exists_exe, $finfo_exe );
-	$exists_exe = Mpp::File::exists_or_can_be_built $finfo_exe = Mpp::File::path_file_info "$name.exe", $mkfile->{CWD}
+	$exists_exe = Mpp::File::exists_or_can_be_built $finfo_exe = Mpp::File::path_file_info( "$name.exe", $mkfile->{CWD} ), 0
 	  if !$exists ||
 	    $_[3] && Mpp::File::stat_exe_separate ? !exists $finfo->{xEXISTS} : !open my $fh, '<', absolute_filename $finfo;
 				# Check for exe, but don't bother returning it, unless full path wanted.
@@ -509,10 +508,10 @@ sub f_find_program {
       }
       next unless $dir;
       my $finfo = file_info $name, $dir;
-      my $exists = Mpp::File::exists_or_can_be_built $finfo, undef, undef, 1;
+      my $exists = Mpp::File::exists_or_can_be_built $finfo, 0, undef, 1;
       if( Mpp::is_windows && $name !~ /\.exe$/ ) {
 	my( $exists_exe, $finfo_exe );
-	$exists_exe = Mpp::File::exists_or_can_be_built $finfo_exe = file_info( "$name.exe", $dir ), undef, undef, 1
+	$exists_exe = Mpp::File::exists_or_can_be_built $finfo_exe = file_info( "$name.exe", $dir ), 0, undef, 1
 	  if !$exists ||
 	    $_[3] && Mpp::File::stat_exe_separate ? !exists $finfo->{xEXISTS} : !open my $fh, '<', absolute_filename $finfo;
 				# Check for exe, but don't bother returning it, unless full path wanted.
@@ -573,7 +572,7 @@ sub f_find_upwards {
     my $dirinfo = $cwd;
     while( 1 ) {
       my $finfo = file_info $_, $dirinfo;
-      if( Mpp::File::exists_or_can_be_built $finfo ) { # Found file in the path?
+      if( Mpp::File::exists_or_can_be_built $finfo, 0 ) { # Found file in the path?
 	$found = 1;
 	push @ret_names, relative_filename $finfo, $cwd;
 	last;			# done searching
@@ -612,7 +611,7 @@ sub f_find_first_upwards {
       return $_[3] ? $finfo : relative_filename $finfo, $cwd
 	if ref $_[3] ?
 	  file_exists $finfo :
-	  Mpp::File::exists_or_can_be_built $finfo; # Found file in the path?
+	  Mpp::File::exists_or_can_be_built $finfo, 0; # Found file in the path?
     }
     last if $dirinfo == ($dirinfo->{ROOT} || 0);
     last unless $dirinfo = $dirinfo->{'..'}; # Look in all directories above us.
@@ -642,8 +641,8 @@ sub f_firstword {
 # environments.
 #
 sub f_first_available {
-  foreach my $fname (split ' ', &arg) {
-    Mpp::File::exists_or_can_be_built( file_info $fname, $_[1]->{CWD} ) and return $fname;
+  for my $fname ( split ' ', &arg ) {
+    Mpp::File::exists_or_can_be_built file_info( $fname, $_[1]->{CWD} ), 0 and return $fname;
   }
   '';
 }
@@ -739,7 +738,7 @@ sub f_infer_objects {
   for my $candidate_obj (map Mpp::Glob::zglob_fileinfo_atleastone($_, $build_cwd), split ' ', $candidate_list) {
 				# Get a list of all the possible objs.
     push @{$candidate_objs{$1}}, $candidate_obj
-      if $candidate_obj->{NAME} =~ /(.+?)(?:\.[^.]+$)?/; # Without the extension.
+      if $candidate_obj->{NAME} =~ /(.+?)(?:\.[^.]+)?$/; # Without the extension.
   }
 #
 # Now look at the list of all the include files.  This is a little tricky
@@ -757,7 +756,7 @@ sub f_infer_objects {
     if $Mpp::log_level;
 
   for( @deps ) {
-    $_->{NAME} =~ /(.+?)(?:\.[^.]+$)?/; # Without the extension.
+    $_->{NAME} =~ /(.+?)(?:\.[^.]+)?$/; # Without the extension.
     $source_names{$1} = 1;	# Indicate that we already have this as a source file.
   }
 
@@ -783,11 +782,11 @@ sub f_infer_objects {
 				# Skip if an error occurred.
 	for( split /\01/, Mpp::File::build_info_string($o_info,'SORTED_DEPS') || '' ) {
 				# Get the list of source files that went into it.
-	  my( $name ) = /([^\/]+?)(?:\.[^\/.]+$)?/; # Without the path or extension.
+	  my( $name ) = /([^\/]+?)(?:\.[^\/.]+)?$/; # Without the path or extension.
 	  if( $candidate_objs{$name} && !$source_names{$name}++ ) { # Newly found source?
 	    if( $candidate_objs{$name}[1] ) { # More than 1 match?
-	      Mpp::print_error( '`', $mkfile_line, "' in infer_objects: more than one possible object for include file $_:",
-				map "\n  " . absolute_filename( $_ ), @{$candidate_objs{$name}} );
+	      warn "$mkfile_line: in infer_objects: more than one possible object for include file $_:",
+		  map( "\n  " . absolute_filename( $_ ), @{$candidate_objs{$name}} ), "\n";
 	    } else { # Found a file?
 	      Mpp::log INFER_DEP => $candidate_objs{$name}[0], $_
 		if $Mpp::log_level;
@@ -920,12 +919,12 @@ sub f_notdir {
 # Return only the files in the list that are actually targets of some rule:
 #
 sub f_only_targets {
-  my $phony = $_[3];
+  my $phony = $_[3] || 0;
   my $cwd = $_[1] && $_[1]{CWD};
   my @ret_files;
 
-  foreach (split ' ', &arg) {
-    foreach my $finfo (zglob_fileinfo($_, $cwd, 0, $phony)) {
+  for my $candidate ( split ' ', &arg ) {
+    for my $finfo ( zglob_fileinfo $candidate, $cwd, 0, $phony ) {
       $phony || exists($finfo->{RULE}) and
 	push @ret_files, relative_filename $finfo, $cwd;
     }
@@ -938,7 +937,7 @@ sub f_only_targets {
 # Return only the targets in the list that are phony:
 #
 sub f_only_phony_targets {
-  $_[3] = \1;
+  $_[3] = 1;
   goto &f_only_targets;
 }
 
@@ -949,9 +948,9 @@ sub f_only_nontargets {
   my $cwd = $_[1] && $_[1]{CWD};
   my @ret_files;
 
-  foreach (split ' ', &arg) {
-    foreach my $finfo (Mpp::Glob::zglob_fileinfo_atleastone($_, $cwd)) {
-      exists($finfo->{RULE}) or
+  for my $candidate ( split ' ', &arg ) {
+    for my $finfo ( Mpp::Glob::zglob_fileinfo_atleastone $candidate, $cwd, undef, 0 ) {
+      exists $finfo->{RULE} or
 	push @ret_files, relative_filename $finfo, $cwd;
     }
   }
@@ -968,9 +967,9 @@ sub f_only_generated {
   my $cwd = $_[1] && $_[1]{CWD};
   my @ret_files;
 
-  foreach (split ' ', &arg) {
-    foreach my $finfo (Mpp::Glob::zglob_fileinfo_atleastone($_, $cwd, 0,0,1)) {
-      Mpp::File::was_built_by_makepp( $finfo ) and
+  for my $candidate ( split ' ', &arg ) {
+    for my $finfo ( Mpp::Glob::zglob_fileinfo_atleastone $candidate, $cwd, 0, 0, 1 ) {
+      Mpp::File::was_built_by_makepp $finfo and
 	push @ret_files, relative_filename $finfo, $cwd;
     }
   }
@@ -986,9 +985,9 @@ sub f_only_stale {
   my $cwd = $_[1] && $_[1]{CWD};
   my @ret_files;
 
-  foreach (split ' ', &arg) {
-    foreach my $finfo (Mpp::Glob::zglob_fileinfo_atleastone($_, $cwd, 0,0,1)) {
-      Mpp::File::is_stale( $finfo ) and
+  for my $candidate ( split ' ', &arg ) {
+    for my $finfo ( Mpp::Glob::zglob_fileinfo_atleastone $candidate, $cwd, 0, 0, 1 ) {
+      Mpp::File::is_stale $finfo and
 	push @ret_files, relative_filename $finfo, $cwd;
     }
   }
@@ -1110,7 +1109,7 @@ sub f_shell {
       $shell_output = `@cmd`;
     }
     $? == 0 or
-      warn "shell command `$str' returned `$?' at `$mkfile_line'\n";
+      warn "$mkfile_line: shell command `$str' returned `$?'\n";
   } else {
 #
 # We used to use Perl's backquotes operators but these seem to have trouble,
@@ -1135,7 +1134,7 @@ sub f_shell {
       exec format_exec_args $str;
       die $_[2] || 'somewhere', ": exec $str failed--$!\n";
     }, ERROR => sub {
-      warn "shell command `$str' returned `$_[0]' at `$mkfile_line'\n";
+      warn "$mkfile_line: shell command `$str' returned `$_[0]'\n";
     };
 
     close $pout;		# In parent, get rid of the output handle.
@@ -1169,7 +1168,7 @@ sub f_sort {
 
 sub f_stem {
   unless( defined $rule ) {
-    warn "\$(stem) or \$* used outside of rule at `$_[2]'\n";
+    warn "$_[2]: \$(stem) or \$* used outside of rule\n";
     return '';
   }
   defined $rule->{PATTERN_STEM} and
@@ -1240,7 +1239,7 @@ sub f_words {
 #
 sub f_target {
   unless( defined $rule ) {
-    warn "\$(output), \$(target) or \$\@ used outside of rule at `$_[2]'\n";
+    warn "$_[2]: \$(output), \$(target) or \$\@ used outside of rule\n";
     return '';
   }
   my $arg = defined $_[0] ? &arg : 0;
@@ -1251,7 +1250,7 @@ sub f_target {
 
 sub f_targets {
   unless( defined $rule ) {
-    warn "\$(outputs) or \$(targets) used outside of rule at `$_[2]'\n";
+    warn "$_[2]: \$(outputs) or \$(targets) used outside of rule\n";
     return '';
   }
   my $arg = defined $_[0] ? &arg : 0;
@@ -1264,7 +1263,7 @@ sub f_targets {
 
 sub f_dependency {
   unless( defined $rule ) {
-    warn "\$(dependency) or \$(input) or \$< used outside of rule at `$_[2]'\n";
+    warn "$_[2]: \$(dependency) or \$(input) or \$< used outside of rule\n";
     return '';
   }
   my $arg = defined $_[0] ? &arg : 0;
@@ -1277,7 +1276,7 @@ sub f_dependency {
 
 sub f_dependencies {
   unless( defined $rule ) {
-    warn "\$(dependencies) or \$(inputs) or \$^ used outside of rule at `$_[2]'\n";
+    warn "$_[2]: \$(dependencies) or \$(inputs) or \$^ used outside of rule\n";
     return '';
   }
   my $arg = defined $_[0] ? &arg : 0;
@@ -1295,7 +1294,7 @@ sub f_dependencies {
 #
 sub f_changed_inputs {
   unless( defined $rule && defined $rule->{EXPLICIT_TARGETS} ) {
-    warn "\$(changed_dependencies) or \$(changed_inputs) or \$? used outside of rule at `$_[2]'\n";
+    warn "$_[2]: \$(changed_dependencies) or \$(changed_inputs) or \$? used outside of rule\n";
     return '';
   }
   my @changed_dependencies =
@@ -1313,7 +1312,7 @@ sub f_changed_inputs {
 
 sub f_sorted_dependencies {
   unless( defined $rule ) {
-    warn "\$(sorted_dependencies) or \$(sorted_inputs) or \$+ used outside of rule at `$_[2]'\n";
+    warn "$_[2]: \$(sorted_dependencies) or \$(sorted_inputs) or \$+ used outside of rule\n";
     return '';
   }
   Mpp::Subs::f_sort join ' ', relative_filenames $rule->{EXPLICIT_DEPENDENCIES};
@@ -1367,7 +1366,7 @@ sub f_foreach {
 }
 
 sub f_warning {
-  warn &arg." at `$_[2]'\n";	# Print the text.
+  warn "$_[2]: " . &arg ."\n";	# Print the text.
   '';
 }
 
@@ -1479,20 +1478,15 @@ sub s_build_check {#_
 sub s_no_implicit_load {
   my ($text_line, $mkfile, $mkfile_line) = @_; # Name the arguments.
 
-  $text_line = $mkfile->expand_text($text_line, $mkfile_line);
   my $cwd = $rule ? $rule->build_cwd : $mkfile->{CWD};
 				# Get the default directory.
 
   local $Mpp::implicitly_load_makefiles; # Temporarily turn off makefile
 				# loading for the expansion of this wildcard.
-
-  my @dirs = map zglob_fileinfo($_, $cwd),
-    split ' ', $mkfile->expand_text($text_line, $mkfile_line);
-				# Get a list of things matching the wildcard.
-  foreach my $dir (@dirs) {
-    undef $dir->{xNO_IMPLICIT_LOAD} if is_or_will_be_dir $dir;
+  wildcard_do {
+    undef $_[0]{xNO_IMPLICIT_LOAD} if is_or_will_be_dir $_[0];
 				# Tag them so they don't load later.
-  }
+  } \2, split ' ', $mkfile->expand_text( $text_line, $mkfile_line );
 }
 
 #
@@ -1710,7 +1704,7 @@ sub prebuild {
     # which Makefiles were loaded. Note that this warning isn't guaranteed to
     # show up when it's called for, because targets that are built via direct
     # calls to Mpp::build() don't undergo this check.
-    warn 'Attempting to build ' . &absolute_filename . " before its makefile is completely loaded\n"
+    warn $mkfile_line, ': Attempting to build ' . &absolute_filename . " before its makefile is completely loaded\n"
       unless ref( $myrule ) eq 'Mpp::DefaultRule' ||
 	exists $finfo->{BUILD_HANDLE} ||
 	$myrule->makefile == $mkfile ||
