@@ -2,7 +2,7 @@
 #
 # This script does the pod to html generation with some heavy massageing.
 #
-# $Id: html.pl,v 1.16 2013/10/13 19:42:43 pfeiffer Exp $
+# $Id: html.pl,v 1.18 2015/08/09 16:24:51 pfeiffer Exp $
 #
 
 #use Pod::Html ();		# Stole module to have consistent id-attributes.
@@ -161,6 +161,7 @@ sub pod2html {
 	if ($Podfile and $Podfile ne '-') {
 	    open $pod, '<', $Podfile
 		or die "$0: cannot open $Podfile file for input: $!\n";
+	    binmode $pod, ':utf8' if $] > 5.008; # suppress warning
 	} else {
 	    open $pod, '-';
 	}
@@ -222,6 +223,7 @@ sub pod2html {
     if($Htmlfile and $Htmlfile ne '-') {
         open $html, ">", $Htmlfile
             or die "$0: cannot open $Htmlfile file for output: $!\n";
+	binmode $html, ':utf8' if $] > 5.008; # suppress warning
     } else {
         open $html, ">-";
     }
@@ -2162,7 +2164,7 @@ sub frame($$$) {
       "</ul></li>";
   }
   my $ret;
-  for my $piece ( split /(div id='_main'>|<pre>.*?<\/pre>)/s, "<?xml version='1.0'?>
+  for my $piece ( split /(main>|<pre>.*?<\/pre>)/s, "<?xml version='1.0'?>
 <!DOCTYPE html>
  <html xmlns='http://www.w3.org/1999/xhtml'>
 <head>
@@ -2179,7 +2181,7 @@ sub frame($$$) {
 </head>
 
 <body>
-<div id='_head'>
+<header>
   <form action='http://www.google.com/search'>
     <p>
       <input type='hidden' name='as_sitesearch' value='makepp.sourceforge.net$sitesearch'/>
@@ -2189,14 +2191,16 @@ sub frame($$$) {
   </form>
   <a title='Makepp Homepage' href='$webroot'><img src='${docroot}makepp.png' alt='Makepp'/></a>
   <div>$tabs</div>
-</div>
+</header>
+<nav>
 <ul>
   <li id='_close'>
     <script type='text/javascript'>r()</script>
-    <span title='Flip side' onclick='lr(this)'>\x{2194}</span><span title='Collapse' onclick='roll(this)'>\x{2013}</span><span title='Expand' onclick='roll(this,1)'>\xa4</span><span title='Close' onclick='nonav(this)'>\xd7</span>
+    <span title='Flip side' onclick='lr()'>\x{2194}</span><span title='Collapse' onclick='roll()'>\x{268a}</span><span title='Expand' onclick='roll(true)'>\x{2752}</span><span title='Close' onclick='nonav()'>\xd7</span>
   </li>$nav
 </ul>
-<div id='_main'>$body</div><div id='_clear'/></body></html>" ) {
+</nav>
+<main>$body</main><div id='_clear'/></body></html>" ) {
     if( !$ret ) {		# compact html
       ($ret = $piece) =~ s/>\s+</></g;
     } else {
@@ -2293,6 +2297,7 @@ sub highlight_variables() {
 
 sub simplify($$$) {
   open my( $tmpfile ), $_[0] or die;
+  binmode $tmpfile, ':utf8' if $] > 5.008; # suppress warning
   my $base = $_[1];
   my $timestamp = $_[2];
   my $file = "$base.html";
@@ -2453,6 +2458,7 @@ sub simplify($$$) {
     s!\./\./makepp!makepp!g;
     $contents .= $_;
   }
+  $contents =~ s!</pre>\n<pre>(.*?)(?=\n|</pre>)!\n$1<b class="tall"> </b>!g;
 
   if( $timestamp || $author ) {
     $timestamp = ($author ? '<br/>' : '') .
